@@ -2,9 +2,9 @@ import logging
 from collections import namedtuple, defaultdict
 from enum import Enum
 from itertools import product
-from random import randint
 from gym import Env
 import gym
+from gym.utils import seeding
 import numpy as np
 
 
@@ -66,9 +66,17 @@ class ForagingEnv(Env):
     )  # reward is available only if is_self
 
     def __init__(
-        self, players, max_player_level, field_size, max_food, sight, max_episode_steps
+        self,
+        players,
+        max_player_level,
+        field_size,
+        max_food,
+        sight,
+        max_episode_steps,
+        force_coop,
     ):
         self.logger = logging.getLogger(__name__)
+        self.seed()
         self.players = [Player() for _ in range(players)]
 
         self.field = np.zeros(field_size, np.int32)
@@ -77,15 +85,20 @@ class ForagingEnv(Env):
         self._food_spawned = 0.0
         self.max_player_level = max_player_level
         self.sight = sight
+        self.force_coop = force_coop
         self._game_over = None
 
-        self.action_space = gym.spaces.Discrete(6)
-        self.observation_space = self._get_observation_space()
+        self.action_space = [gym.spaces.Discrete(6)] * len(self.players)
+        self.observation_space = [self._get_observation_space()] * len(self.players)
 
         self._rendering_initialized = False
         self._valid_actions = None
         self._max_episode_steps = max_episode_steps
 
+    def seed(self, seed=None):
+        self.np_random, seed = seeding.np_random(seed)
+        return [seed]
+
     def _get_observation_space(self):
         """The Observation Space for each agent.
         - all of the board (board_size^2) with foods
@@ -149,12 +162,21 @@ class ForagingEnv(Env):
             for player in self.players
         }
 
-    def neighborhood(self, row, col, distance=1):
+    def neighborhood(self, row, col, distance=1, ignore_diag=False):
+        if not ignore_diag:
+            return self.field[
+                max(row - distance, 0) : min(row + distance + 1, self.rows),
+                max(col - distance, 0) : min(col + distance + 1, self.cols),
+            ]
 
-        return self.field[
-            max(row - distance, 0) : min(row + distance + 1, self.rows),
-            max(col - distance, 0) : min(col + distance + 1, self.cols),
-        ]
+        return (
+            self.field[
+                max(row - distance, 0) : min(row + distance + 1, self.rows), col
+            ].sum()
+            + self.field[
+                row, max(col - distance, 0) : min(col + distance + 1, self.cols)
+            ].sum()
+        )
 
     def adjacent_food(self, row, col):
         return (
@@ -188,19 +210,26 @@ class ForagingEnv(Env):
 
         food_count = 0
         attempts = 0
+        min_level = max([p.level for p in self.players]) + 1 if self.force_coop else 1
 
         while food_count < max_food and attempts < 1000:
             attempts += 1
-            row = randint(1, self.rows - 2)
-            col = randint(1, self.cols - 2)
+            row = self.np_random.randint(1, self.rows - 1)
+            col = self.np_random.randint(1, self.cols - 1)
 
             # check if it has neighbors:
-            if self.neighborhood(
-                row, col, distance=2
-            ).sum() > 0 or not self._is_empty_location(row, col):
+            if (
+                self.neighborhood(row, col).sum() > 0
+                or self.neighborhood(row, col, distance=2, ignore_diag=True) > 0
+                or not self._is_empty_location(row, col)
+            ):
                 continue
 
-            self.field[row, col] = randint(1, max_level)
+            self.field[row, col] = (
+                min_level
+                if min_level == max_level
+                else self.np_random.randint(min_level, max_level)
+            )
             food_count += 1
         self._food_spawned = self.field.sum()
 
@@ -222,11 +251,13 @@ class ForagingEnv(Env):
             player.reward = 0
 
             while attempts < 1000:
-                row = randint(0, self.rows - 1)
-                col = randint(0, self.cols - 1)
+                row = self.np_random.randint(0, self.rows - 1)
+                col = self.np_random.randint(0, self.cols - 1)
                 if self._is_empty_location(row, col):
                     player.setup(
-                        (row, col), randint(1, max_player_level), self.field_size
+                        (row, col),
+                        self.np_random.randint(1, max_player_level),
+                        self.field_size,
                     )
                     break
                 attempts += 1
@@ -299,7 +330,7 @@ class ForagingEnv(Env):
 
     def _make_gym_obs(self, observations):
         def make_obs_array(observation):
-            obs = np.zeros(self.observation_space.shape)
+            obs = np.zeros(self.observation_space[0].shape)
             # obs[: observation.field.size] = observation.field.flatten()
             for i in range(self.max_food):
                 obs[3 * i] = -1
@@ -417,7 +448,9 @@ class ForagingEnv(Env):
 
             # else the food was loaded and each player scores points
             for a in adj_players:
-                a.reward = float(food) / self._food_spawned  # normalize reward
+                a.reward = float(a.level * food) / float(
+                    adj_player_level * self._food_spawned
+                )  # normalize reward
             # and the food is removed
             self.field[frow, fcol] = 0
 
